% NGLIGEN    Ng and Li's General Test-Problem Generator.
%
% NGLIGEN uses the parameters stored in "nglinnnn.par" to generate an
% instant of Ng and Li's test problems.
% Here nnnn is the problem-ID between 1000 and 9999.
% It is also the random seed for generating the problem.
%
% The file "nglinnnn.par" must contain the following items and 
% appear in the following order:
% - n (= the number of decision variables and the number of subproblems)
%   It is a positive integer.
% - nDiff (= the number of relatively difficult subproblems)
%   It is a positive integer not greater than n.
%   The remaining (n - nDiff) subproblems are then relatively easy 
%   subproblems.
% - aLB and aUB (= lower and upper bounds for a(i), respectively)
%   In theory, a(i) > 0.
%   In practice, we generate a(i) uniformly in [aLB, aUB], 
%   where 0 < aLB < aUB.
% - pLB and pUB (= lower and upper bounds for p(i), respectively)
%   In theory, p(i) is a real number.
%   In practice, we generate p(i) uniformly in [pLB, pUB], 
%   where pLB < pUB.
% - qLB and qPer
%   In thoery, q(i) is a real number such that q(i) < p(i)^2
%   In practice, we generate q(i) uniformly in [qLB, qUB], where
%      qLB <= 0,
%      qUB = qPer*(p(i)^2), and qPer in (0,1).
% - alphaPer
%   In theory,
%      -p(i)-2*r(i) < alpha(i) < -p(i)-r(i)*sqrt(3)  OR
%      -p(i)+r(i)*sqrt(3) < alpha(i) < -p(i)+2*r(i)
%   In practice, to avoid the boundary points, we generate alpha(i)
%   uniformly in the middle alphaPer (e.g. 0.95) of the open intervals.
% - dLB and dUB
%   In theory, d(i) > 0.
%   In practice, we generate d(i) uniformly in [dLB, dUB], 
%   where 0 < dLB < dUB.
% - wLB and wUB
%   If wLB and wUB are present, then Algorithm 4 will be used, 
%   otherwise Algorithm 2 will be used.
%   In theory, wLB and wUB are real numbers.
% - deltaLB and deltaUB
%   deltaLB and deltaUB will be loaded unless wLB and wUB are present.
%   In theory deltaL(i) > 0 and deltaU(i) > 0
%   In practice, we generate deltaL(i) and deltaU(i) uniformly in 
%   [deltaLB, deltaUB], where 0 < deltaLB < deltaUB.





function ngligen(ID)


if nargin == 0
  error('ngligen(ID), where 1000 <= ID <= 9999');
elseif (ID < 1000) || (ID > 9999)
  error(['To generate a user-defined problem, ',...
         'please use an ID between 1000 and 9999.']);
else
  NL = read_para(ID);
  fprintf('Generating user-defined test-problem number %d... ', NL.id);
  generate_rand(NL);
  nglimain(NL);
  fprintf('Done.\n');
end





function NL = read_para(ID)


NL.id = ID;

fid = fopen(['ngli', num2str(NL.id,'%04u'), '.par'], 'r');
if fid <= 0
  error(['The parameter file "ngli', num2str(NL.id,'%04u'),...
         '.par" does not exist.']);
end


% Read n (>= 1)
%
NL.n = fscanf(fid, '%d', 1);
if NL.n < 2
  fclose(fid);
  error('Dimension n >= 1.');
end


% Read nDiff
%
NL.nDiff = fscanf(fid, '%d', 1);
if (NL.nDiff < 0) || (NL.nDiff > NL.n)
  fclose(fid);
  error('Number of difficulty subproblems in [0,n].');
end


% Read aLB and aUB
%
NL.aLB = fscanf(fid, '%f', 1);
NL.aUB = fscanf(fid, '%f', 1);
if NL.aLB >= NL.aUB
  fclose(fid);
  error('Lower bound of a(i) must less than its upper bound.');
end


% Read pLB and pUB
%
NL.pLB = fscanf(fid, '%f', 1);
NL.pUB = fscanf(fid, '%f', 1);
if NL.pLB >= NL.pUB
  fclose(fid);
  error('Lower bound of p(i) must less than its upper bound.');
end


% Read qLB and qPer
%
NL.qLB  = fscanf(fid, '%f', 1);
NL.qPer = fscanf(fid, '%f', 1);
if NL.qLB > 0
  fclose(fid);
  error('Lower bound of q(i) <= 0.');
elseif (NL.qPer <= 0) || (NL.qPer >= 1) 
  fclose(fid);
  error('The percentage qPer in (0,1)');
end


% Read alphaPer
%
NL.alphaPer = fscanf(fid, '%f', 1);
if (NL.alphaPer < 0.5) || (NL.alphaPer >= 1)
  fclose(fid);
  error('The percentage alphaPer in [0.5,1).');
end


% Read dLB and dUB
%
NL.dLB = fscanf(fid, '%f', 1);
NL.dUB = fscanf(fid, '%f', 1);
if NL.dLB >= NL.dUB
  fclose(fid);
  error('Lower bound of d(i) must less than its upper bound.');
end


% Read wLB, wUB, deltaLB and deltaUB
%
tmp = fscanf(fid, '%f %f', [2 inf])';
if isempty(tmp)
  NL.wLB     = [];
  NL.wUB     = [];
  NL.deltaLB = [];
  NL.deltaUB = [];
  fclose(fid);
elseif size(tmp,1) == 1
  fclose(fid);
  error('Lower and upper bounds of deltaL(i) and deltaU(i) do not exist.');
elseif size(tmp,1) >= 2
  NL.wLB     = tmp(1,1);
  NL.wUB     = tmp(1,2);
  NL.deltaLB = tmp(2,1);
  NL.deltaUB = tmp(2,2);
  if NL.wLB >= NL.wUB
    fclose(fid);
    error('Lower bound of w(i) must less than its upper bound.');
  end
  if NL.deltaLB >= NL.deltaUB
    fclose(fid);
    error('Lower bounds of deltaL(i) and deltaU(i) must less than their upper bounds.');
  end
end





function generate_rand(NL)

RandStream.setDefaultStream(RandStream('mt19937ar','seed',NL.id));

AllRand = rand(8*NL.n, 1);

fid = fopen(['ngli', num2str(NL.id,'%04u'), '.rn'], 'w');
fprintf(fid, '% 19.15e\n', AllRand);
fclose(fid);
